import datetime
import random
import string
from typing import Literal

from .utils import add_help


@add_help
class Person:
    """用于生成测试数据，所有数据都是随机生成的，仅用于测试目的"""

    def help() -> None: ...

    def __load_area_code_dict(self) -> dict[str, str]:
        """加载区域地址"""
        from .data._internal_utils import data_dict

        return data_dict

    def __birthday(self) -> str:
        """功能：随机生成1930年之后的出生日期"""
        start_date: datetime.datetime = datetime.datetime.strptime(
            "1950-01-01 00:00:00", "%Y-%m-%d %H:%M:%S"
        )
        end_date: datetime.datetime = datetime.datetime.now()
        delta: datetime.timedelta = end_date - start_date
        days: int = delta.days
        rand_day = random.randint(0, days)
        offset = datetime.timedelta(days=rand_day)
        birthday: datetime.datetime = start_date + offset
        return birthday.strftime("%Y%m%d")

    def __check(self, id_num) -> str:
        """生成最后一位校验码"""
        i = 0
        count = 0
        weight: list[int] = [
            7,
            9,
            10,
            5,
            8,
            4,
            2,
            1,
            6,
            3,
            7,
            9,
            10,
            5,
            8,
            4,
            2,
        ]  # 权重项
        check_code: dict[str, str] = {
            "0": "1",
            "1": "0",
            "2": "X",
            "3": "9",
            "4": "8",
            "5": "7",
            "6": "6",
            "7": "5",
            "8": "5",
            "9": "3",
            "10": "2",
        }  # 校验码映射
        for i in range(len(id_num)):
            count: int = count + int(id_num[i]) * weight[i]
        return check_code[str(count % 11)]  # 算出校验码

    def __id_sex_age_area(self) -> tuple[str, Literal["女", "男"], int, str]:
        """生成身份证号码、性别、年龄、区域"""
        area_code_and_name: dict[str, str] = self.__load_area_code_dict()
        rand_area_code: str = random.choice(list(area_code_and_name.keys()))
        area_name: str = area_code_and_name[rand_area_code]

        birthday: str = self.__birthday()  # 生成出生日期
        ord_num: int = random.randint(100, 999)  # 生成顺序号
        sex: int = ord_num % 2  # 生成性别号

        check_code: str = self.__check(f"{rand_area_code}{birthday}{ord_num}")

        # 拼装身份证号
        id_card: str = f"{rand_area_code}{birthday}{ord_num}{check_code}"
        gender: Literal["女"] | Literal["男"] = "女" if sex == 0 else "男"

        # 计算年龄
        birth_date: datetime.datetime = datetime.datetime.strptime(birthday, "%Y%m%d")
        current_date: datetime.datetime = datetime.datetime.now()
        age: int = (
            current_date.year
            - birth_date.year
            - (
                (current_date.month, current_date.day)
                < (birth_date.month, birth_date.day)
            )
        )

        return id_card, gender, age, area_name

    def education(self) -> str:
        """学历、教育经历"""
        return random.choice(
            ["小学", "初中", "高中", "中专", "大专", "本科", "硕士", "研究生", "博士"]
        )

    def id_type(self) -> str:
        """证件类型"""
        return random.choice(
            [
                "身份证",
                "居民身份证",
                "士官证",
                "军官证",
                "学生证",
                "驾驶证",
                "驾照",
                "护照",
                "港澳通行证",
                "营业执照",
                "组织机构代码证",
                "台胞证",
                "文职干部证",
                "部队离退休证",
                "香港特区护照/身份证明",
                "澳门特区护照/身份证明",
                "台湾居民来往大陆通行证",
                "境外永久居住证",
                "户口薄",
            ]
        )

    def marital(self) -> str:
        """婚姻状态"""
        return random.choice(["未婚", "已婚", "离婚", "丧偶", "离异"])

    def passport_number(self) -> str:
        """护照号"""
        return "D" + "".join(random.choices("0123456789", k=8))

    def sex(self) -> str:
        """性别"""
        return random.choice(["男", "女"])

    def address(self) -> str:
        """地址"""
        from .data._addr import addr_info_list

        return random.choice(addr_info_list)

    def province(self, long: bool = False) -> str:
        """省份，简写和全称"""
        name = [
            "北京市",
            "天津市",
            "河北省",
            "山西省",
            "内蒙古自治区",
            "辽宁省",
            "吉林省",
            "黑龙江省",
            "上海市",
            "江苏省",
            "浙江省",
            "安徽省",
            "福建省",
            "江西省",
            "山东省",
            "河南省",
            "湖北省",
            "湖南省",
            "广东省",
            "广西壮族自治区",
            "海南省",
            "重庆市",
            "四川省",
            "贵州省",
            "云南省",
            "西藏自治区",
            "陕西省",
            "甘肃省",
            "青海省",
            "宁夏回族自治区",
            "新疆维吾尔自治区",
            "台湾省",
            "香港特别行政区",
            "澳门特别行政区",
        ]
        short_name = [
            "北京",
            "天津",
            "河北",
            "山西",
            "内蒙古",
            "辽宁",
            "吉林",
            "黑龙江",
            "上海",
            "江苏",
            "浙江",
            "安徽",
            "福建",
            "江西",
            "山东",
            "河南",
            "湖北",
            "湖南",
            "广东",
            "广西",
            "海南",
            "重庆",
            "四川",
            "贵州",
            "云南",
            "西藏",
            "陕西",
            "甘肃",
            "青海",
            "宁夏",
            "新疆",
            "台湾",
            "香港",
            "澳门",
        ]
        if long:
            return random.choice(name)
        else:
            return random.choice(short_name)

    def store_name(self) -> str:
        """店铺名称"""
        from .data._shop_sign import shop_sign_list

        return random.choice(shop_sign_list)

    def id_number(self) -> str:
        """身份证号"""
        return self.__id_sex_age_area()[0]

    def mac_address(self, symbol: Literal[":", "-"] = ":") -> str:
        """MAC地址"""
        mac_parts: list[int] = [random.randint(0x00, 0xFF) for _ in range(6)]
        mac_address: str = symbol.join(["%02x" % part for part in mac_parts])
        return mac_address

    def phone_number(self) -> int:
        """手机号"""
        prefix = random.choice(
            [
                "130",
                "131",
                "132",
                "133",
                "134",
                "135",
                "136",
                "137",
                "138",
                "139",
                "150",
                "151",
                "152",
                "153",
                "155",
                "156",
                "157",
                "158",
                "159",
                "176",
                "180",
                "181",
                "182",
                "183",
                "184",
                "185",
                "186",
                "187",
                "188",
                "189",
            ]
        )
        # 随机生成电话号的后缀
        suffix: str = "".join(
            random.choice(["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"])
            for _ in range(8)
        )
        return int(prefix + suffix)

    def landline_number(self) -> str:
        """座机号"""
        prefix: list[str] = [
            "010",
            "022",
            "0311",
            "0315",
            "0335",
            "0310",
            "0319",
            "0312",
            "0313",
            "0314",
            "0317",
            "0316",
            "0318",
            "0351",
            "0352",
            "0353",
            "0355",
            "0356",
            "0349",
            "0354",
            "0359",
            "0350",
            "0357",
            "0358",
            "0471",
            "0472",
            "0473",
            "0476",
            "0475",
            "0477",
            "0470",
            "0478",
            "0474",
            "0482",
            "0479",
            "0483",
            "024",
            "0411",
            "0412",
            "0413",
            "0414",
            "0415",
            "0416",
            "0417",
            "0418",
            "0419",
            "0427",
            "0410",
            "0421",
            "0429",
            "0431",
            "0423",
            "0434",
            "0437",
            "0435",
            "0439",
            "0438",
            "0436",
            "0433",
            "0451",
            "0452",
            "0467",
            "0468",
            "0469",
            "0459",
            "0458",
            "0454",
            "0464",
            "0453",
            "0456",
            "0455",
            "0457",
            "021",
            "025",
            "0510",
            "0516",
            "0519",
            "0512",
            "0513",
            "0518",
            "0517",
            "0515",
            "0514",
            "0511",
            "0523",
            "0527",
            "0571",
            "0574",
            "0577",
            "0573",
            "0572",
            "0575",
            "0579",
            "0570",
            "0580",
            "0576",
            "0578",
            "0551",
            "0553",
            "0552",
            "0554",
            "0555",
            "0561",
            "0562",
            "0556",
            "0559",
            "0550",
            "0558",
            "0557",
            "0564",
            "0566",
            "0563",
            "0591",
            "0592",
            "0594",
            "0598",
            "0595",
            "0596",
            "0599",
            "0597",
            "0593",
            "0791",
            "0798",
            "0799",
            "0792",
            "0790",
            "0701",
            "0797",
            "0796",
            "0795",
            "0794",
            "0793",
            "0531",
            "0532",
            "0533",
            "0632",
            "0546",
            "0535",
            "0536",
            "0537",
            "0538",
            "0631",
            "0633",
            "0539",
            "0534",
            "0635",
            "0543",
            "0530",
            "0371",
            "0378",
            "0379",
            "0375",
            "0372",
            "0392",
            "0373",
            "0391",
            "0393",
            "0374",
            "0395",
            "0398",
            "0377",
            "0370",
            "0376",
            "0394",
            "0396",
            "027",
            "0714",
            "0719",
            "0717",
            "0710",
            "0711",
            "0724",
            "0712",
            "0716",
            "0713",
            "0715",
            "0722",
            "0718",
            "0728",
            "0731",
            "0733",
            "0732",
            "0734",
            "0739",
            "0730",
            "0736",
            "0744",
            "0737",
            "0735",
            "0746",
            "0745",
            "0738",
            "0743",
            "020",
            "0751",
            "0755",
            "0756",
            "0754",
            "0757",
            "0750",
            "0759",
            "0668",
            "0758",
            "0752",
            "0753",
            "0660",
            "0762",
            "0662",
            "0763",
            "0769",
            "0760",
            "0768",
            "0663",
            "0766",
            "0771",
            "0772",
            "0773",
            "0774",
            "0779",
            "0770",
            "0777",
            "0775",
            "0776",
            "0778",
            "0898",
            "023",
            "028",
            "0813",
            "0812",
            "0830",
            "0838",
            "0816",
            "0839",
            "0825",
            "0832",
            "0833",
            "0817",
            "0831",
            "0826",
            "0818",
            "0835",
            "0827",
            "0873",
            "0836",
            "0834",
            "0851",
            "0858",
            "0852",
            "0853",
            "0857",
            "0856",
            "0859",
            "0855",
            "0854",
            "0871",
            "0874",
            "0877",
            "0875",
            "0870",
            "0888",
            "0879",
            "0883",
            "0878",
            "0876",
            "0691",
            "0872",
            "0692",
            "0886",
            "0887",
            "0891",
            "0892",
            "0895",
            "0894",
            "0893",
            "0896",
            "0897",
            "029",
            "0919",
            "0917",
            "0910",
            "0913",
            "0911",
            "0916",
            "0912",
            "0915",
            "0914",
            "0931",
            "0937",
            "0935",
            "0943",
            "0938",
            "0936",
            "0933",
            "0934",
            "0932",
            "0939",
            "0930",
            "0941",
            "0971",
            "0972",
            "0970",
            "0973",
            "0974",
            "0975",
            "0976",
            "0977",
            "0951",
            "0952",
            "0953",
            "0954",
            "0955",
            "0991",
            "0990",
            "0995",
            "0902",
            "0994",
            "0909",
            "0996",
            "0997",
            "0908",
            "0998",
            "0903",
            "0999",
            "0901",
            "0906",
            "0993",
            "0992",
            "8863",
            "8862",
            "88649",
            "8865",
            "8864",
            "8866",
            "88689",
            "8868",
            "8867",
            "00852",
            "00853",
        ]
        rand_prefix: str = random.choice(prefix)
        return f"{rand_prefix}-{random.choice(string.digits[1:])}{''.join(random.choices(string.digits,k=7))}"

    def latitude(self) -> float:
        """纬度 N（北） 表示北半球。S（南） 表示南半球。"""
        return f"{round(random.uniform(-90, 90), 6)}°{random.choice(['N','S'])}"

    def longitude(self) -> float:
        """经度 E（东） 表示东半球。W（西） 表示西半球。"""
        return f"{round(random.uniform(-180, 180), 6)}°{random.choice(['E','W'])}"

    def occupation(self) -> str:
        """职业"""
        return random.choice(
            [
                "教师",
                "工人",
                "记者",
                "演员",
                "作曲家",
                "架构师",
                "营养师",
                "鼓手",
                "厨师",
                "医生",
                "护士",
                "司机",
                "军人",
                "律师",
                "商人",
                "会计",
                "店员",
                "出纳员",
                "作家",
                "导游",
                "警察",
                "歌手",
                "画家",
                "裁缝",
                "翻译官",
                "法官",
                "保安",
                "花匠",
                "服务员",
                "清洁工",
                "建筑师",
                "理发师",
                "采购员",
                "消防员",
                "机修工",
                "推销员",
                "魔术师",
                "模特",
                "邮递员",
                "售货员",
                "救生员",
                "运动员",
                "工程师",
                "飞行员",
                "管理员",
                "机械师",
                "经纪人",
                "歌唱家",
                "审计员",
                "漫画家",
                "园艺师",
                "科学家",
                "主持人",
                "调酒师",
                "化妆师",
                "艺术家",
                "糕点师",
                "甜品师",
                "外交官",
                "舞蹈家",
                "箭术",
                "溜冰",
                "弹钢琴",
                "古筝手",
                "钢琴家",
                "设计师",
                "CEO",
                "机 长",
                "记者",
                "赛车手",
                "教练",
                "兽医",
                "特警",
                "按摩",
            ]
        )

    def school(self) -> str:
        """学校名称"""
        return random.choice(
            [
                "麻省理工学院",
                "牛津大学",
                "斯坦福大学",
                "剑桥大学",
                "哈佛大学",
                "加州理工学院",
                "帝国理工学院",
                "苏黎世联邦理工大学",
                "伦敦大学学院",
                "芝加哥大学",
            ]
        )

    def full_name(self) -> str:
        """姓名"""
        return f"{self.last_name()}{self.first_name()}"

    def first_name(self) -> str:
        """名，只生成随机名，不包含姓氏"""
        return random.choice(
            [
                "伟",
                "芳",
                "娜",
                "秀英",
                "敏",
                "静",
                "丽",
                "强",
                "磊",
                "军",
                "洋",
                "勇",
                "艳",
                "杰",
                "娟",
                "博",
                "文",
                "涛",
                "慧",
                "明",
                "建国",
                "丽丽",
                "媛",
                "子涵",
                "子轩",
                "浩然",
                "昊然",
                "浩",
                "振华",
            ]
        )

    def last_name(self) -> str:
        """姓，生成随机的姓氏"""
        return random.choice(
            [
                "赵",
                "钱",
                "孙",
                "李",
                "周",
                "吴",
                "郑",
                "王",
                "冯",
                "陈",
                "褚",
                "卫",
                "蒋",
                "沈",
                "韩",
                "杨",
                "朱",
                "秦",
                "尤",
                "许",
                "何",
                "吕",
                "施",
                "张",
                "孔",
                "曹",
                "严",
                "华",
                "金",
                "魏",
                "陶",
                "姜",
                "戚",
                "谢",
                "邹",
                "喻",
                "柏",
                "水",
                "窦",
                "章",
                "云",
                "苏",
                "潘",
                "葛",
                "奚",
                "范",
                "彭",
                "郎",
                "鲁",
                "韦",
                "昌",
                "马",
                "苗",
                "凤",
                "花",
                "方",
                "俞",
                "任",
                "袁",
                "柳",
                "酆",
                "鲍",
                "史",
                "唐",
                "费",
                "廉",
                "岑",
                "薛",
                "雷",
                "贺",
                "倪",
                "汤",
                "滕",
                "殷",
                "罗",
                "毕",
                "郝",
                "邬",
                "安",
                "常",
                "乐",
                "于",
                "时",
                "傅",
                "皮",
                "卞",
                "齐",
                "康",
                "伍",
                "余",
                "元",
                "卜",
                "顾",
                "孟",
                "平",
                "黄",
                "郭",
                "轩辕",
                "公孙",
                "慕容",
                "司马",
            ]
        )

    def religion(self) -> str:
        """宗教信仰"""
        return random.choice(
            [
                "道教",
                "佛教",
                "伊斯兰教",
                "基督教",
                "印度教",
                "犹太教",
                "婆罗门教",
                "耆那教",
            ]
        )

    def age(self, min: int = 16, max: int = 101) -> int:
        """年龄"""
        return random.randint(min, max)

    def ethnicity(self, country: str = "zh") -> str:
        """民族、种族"""
        chinese_ethnicity: list[str] = [
            "汉族",
            "蒙古族",
            "回族",
            "藏族",
            "维吾尔族",
            "苗族",
            "彝族",
            "壮族",
            "布依族",
            "朝鲜族",
            "满族",
            "侗族",
            "瑶族",
            "白族",
            "土家族",
            "哈尼族",
            "哈萨克族",
            "傣族",
            "黎族",
            "傈僳族",
            "佤族",
            "畲族",
            "高山族",
            "拉祜族",
            "水族",
            "东乡族",
            "纳西族",
            "景颇族",
            "柯尔克孜族",
            "土族",
            "达斡尔族",
            "仫佬族",
            "羌族",
            "布朗族",
            "撒拉族",
            "毛南族",
            "仡佬族",
            "锡伯族",
            "阿昌族",
            "普米族",
            "塔吉克族",
            "怒族",
            "乌孜别克族",
            "俄罗斯族",
            "鄂温克族",
            "崩龙族",
            "保安族",
            "裕固族",
            "京族",
            "塔塔尔族",
            "独龙族",
            "鄂伦春族",
            "赫哲族",
            "门巴族",
            "珞巴族",
            "基诺族",
        ]
        match country:
            case "zh":
                return random.choice(chinese_ethnicity)
            case _:
                return random.choice(chinese_ethnicity)

    def residence(self) -> str:
        """小区、住宅"""
        from .data._housing import housing

        return random.choice(housing)

    def country(self, en: bool = False) -> str:
        """国家名"""
        from .data._country import en_country, zh_country

        if en:
            return random.choice(en_country)
        return random.choice(zh_country)

    def industry(self) -> str:
        """行业"""
        return random.choice(
            ["金融", "教育", "通信", "信息安全", "物流", "能源", "医疗", "军工"]
        )

    def license_plate(self, battery: bool = False, symbol: str = "· ") -> str:
        """车牌号"""
        provinces_abbreviations: list[str] = [
            "京",
            "津",
            "沪",
            "渝",
            "冀",
            "晋",
            "蒙",
            "辽",
            "吉",
            "黑",
            "苏",
            "浙",
            "皖",
            "闽",
            "赣",
            "鲁",
            "豫",
            "鄂",
            "湘",
            "粤",
            "桂",
            "琼",
            "川",
            "黔",
            "滇",
            "藏",
            "陕",
            "甘",
            "青",
            "宁",
            "新",
            "港",
            "澳",
            "台",
        ]
        ascii_uppercase = "ABCDEFGHJKLMNPQRSTUVWXYZ"  # I、O
        province_letter: str = random.choice(provinces_abbreviations)
        letter_part: str = random.choice(string.ascii_uppercase)
        license_plate: str = "".join(
            random.choices((string.digits + ascii_uppercase), k=5)
        )

        if battery:
            license_plate: str = "".join(random.choices((string.digits), k=5))
            battery_symbol: str = random.choice(
                "DF"
            )  # D代表纯电动新能源汽车，F代表非纯电动新能源汽车。
            return (
                f"{province_letter}{letter_part}{symbol}{battery_symbol}{license_plate}"
            )

        return f"{province_letter}{letter_part}{symbol}{license_plate }"

    def email(self, prefix=None):
        """邮箱"""
        email_suffix = [
            "@gmail.com",
            "@qq.com",
            "@163.com",
            "@sina.com",
            "@163.com",
            "@outlook.com",
            "@hotmail.com",
            "@yahoo.com",
            "@yahoo.co.uk",
            "@yahoo.ca",
            "@yahoo.fr",
            "@yahoo.de",
            "@yahoo.it",
            "@yahoo.es",
        ]
        if prefix is None:
            prefix = "".join(random.choices(string.ascii_lowercase, k=5))
        return f"{prefix}{random.choice(email_suffix)}"

    def profile(self, zh: bool = False) -> dict:
        """包含随机生成的个人信息的字典，例如姓名、性别、年龄、职业等。"""

        _id_card, _sex, _age, area_code_name = self.__id_sex_age_area()

        labels = {
            "name": "姓名" if zh else "Name",
            "sex": "性别" if zh else "Sex",
            "age": "年龄" if zh else "Age",
            "occupation": "职业" if zh else "Occupation",
            "ethnicity": "民族" if zh else "Ethnicity",
            "school": "学校" if zh else "School",
            "religion": "宗教信仰" if zh else "Religion",
            "phone_number": "手机号" if zh else "Phone Number",
            "landline_number": "座机号" if zh else "Landline Number",
            "address": "地址" if zh else "Address",
            "residence": "居住地" if zh else "Residence",
            "id_card": "身份证" if zh else "ID Card",
            "mac": "MAC",
            "passport_number": "护照号" if zh else "Passport Number",
            "latitude": "经度" if zh else "Longitude",
            "longitude": "纬度" if zh else "Latitude",
            "country": "国家" if zh else "Country",
            "marital": "婚姻状况" if zh else "Marital",
            "email": "邮箱" if zh else "Email",
            "id_type": "证件类型" if zh else "ID Type",
            "education": "学历" if zh else "Education",
            "license_plate": "车牌号" if zh else "License Plate",
        }

        return {
            labels["name"]: self.full_name(),
            labels["sex"]: _sex,
            labels["age"]: _age,
            labels["occupation"]: self.occupation(),
            labels["ethnicity"]: self.ethnicity(),
            labels["school"]: self.school(),
            labels["religion"]: self.religion(),
            labels["phone_number"]: self.phone_number(),
            labels["landline_number"]: self.landline_number(),
            labels["address"]: f"{area_code_name}{self.residence()}",
            labels["residence"]: self.residence(),
            labels["id_card"]: _id_card,
            labels["mac"]: self.mac_address(),
            labels["passport_number"]: self.passport_number(),
            labels["latitude"]: self.latitude(),
            labels["longitude"]: self.longitude(),
            labels["country"]: self.country(),
            labels["marital"]: self.marital(),
            labels["email"]: self.email(),
            labels["id_type"]: self.id_type(),
            labels["education"]: self.education(),
            labels["license_plate"]: self.license_plate(),
        }



from czo import DateLib, DirLib, Faker, NetLib, PathLib, Person, Random


def test_random(capsys):
    Random.help()

    captured = capsys.readouterr()

    assert "Random" in captured.out and "help" in captured.out


def test_person(capsys):
    Person.help()

    captured = capsys.readouterr()

    assert "Person" in captured.out and "help" in captured.out


def test_pathlib(capsys):
    PathLib.help()

    captured = capsys.readouterr()

    assert "PathLib" in captured.out and "help" in captured.out


def test_netlib(capsys):
    NetLib.help()

    captured = capsys.readouterr()

    assert "NetLib" in captured.out and "help" in captured.out


def test_faker(capsys):
    Faker.help()

    captured = capsys.readouterr()

    assert "Faker" in captured.out and "help" in captured.out


def test_dirlib(capsys):
    DirLib.help()

    captured = capsys.readouterr()

    assert "DirLib" in captured.out and "help" in captured.out


def test_datelib(capsys):
    DateLib.help()

    captured = capsys.readouterr()

    assert "DateLib" in captured.out and "help" in captured.out



import inspect
import unicodedata

from .__meta import Singleton


def count_chr_length(text, no_ascii=2) -> int:
    """
    将 ASCII 字符计算为 1 个单位长度。

    将大部分非 ASCII 字符（如汉字、日文、韩文等）计算为 2 个单位长度。
    """
    length = 0
    for c in text:
        if unicodedata.east_asian_width(c) in ["F", "W"]:
            length += no_ascii  # 全角宽度字符
        else:
            length += 1  # 半角宽度字符
    return length


def _get_methods_and_properties_with_docs(cls):
    """通过这个函数，可以方便地获取类的所有公共方法和属性，并以一种格式化的方式输出它们的名称和文档注释。"""
    methods_and_properties: dict[str, str] = {}

    # 获取类的文档注释
    class_doc: str | None = inspect.getdoc(cls)
    if class_doc is not None:
        methods_and_properties["__class__"] = f"C: {class_doc}".split("\n")[0]

    # 获取类的所有成员（包括方法和属性）
    for name, member in inspect.getmembers(cls):
        if name.startswith("_"):  # 过滤掉私有方法
            continue
        elif inspect.isfunction(member):
            prefix = "F: "
        elif isinstance(member, property):
            prefix = "P: "
        elif inspect.ismethod(member):
            prefix = "M: "

        # 提取文档注释的第一行，并为功能加上标签
        methods_and_properties[name] = (
            f"{prefix}{inspect.getdoc(member)}".split("\n")[0]
            if name != "help"
            else f"{prefix}可以方便地获取{cls.__name__}类的所有公共方法和属性，并以一种格式化的方式输出它们的名称和文档注释。"
        )

    max_doc_length: int = 0
    max_method_length: int = 0

    # 计算方法名和文档注释的最大长度
    for method, doc in methods_and_properties.items():
        if len(method) > max_method_length:
            max_method_length = len(method)

        if (length := count_chr_length(doc)) > max_doc_length:
            max_doc_length = length

    # 总长度为方法名最长的长度加上文档注释最长的长度再加上3个单位长度的间隔
    max_length: int = max_doc_length + max_method_length + 3

    print(f" {'-' * max_length}")

    tag: str = f"【{cls.__name__}】 F|P|M 分别表示 函数|属性|方法 。C 是类的文档注释。"
    tag_length: int = count_chr_length(tag)
    print(f"| {tag}{' ' * (max_length-tag_length-2)} |")
    print(f"|{'-'*(max_length)}|")

    # 打印方法和属性及其文档注释
    help_print = None
    for method, doc in methods_and_properties.items():
        info = f"| {method:<{max_method_length}} {doc}"
        while count_chr_length(info) < max_length:
            info += " "
        # 始终在最后输出帮助方法
        if method == "help":
            help_print = info + " |"
            continue
        print(info + " |")
    print(help_print)
    print(f" {'-'*max_length}")


def add_help(cls):
    """通过这个装饰器，可以方便地获取类的所有公共方法和属性，并以一种格式化的方式输出它们的名称和文档注释。"""
    cls.help = lambda: _get_methods_and_properties_with_docs(cls)
    return cls



import ipaddress
from ipaddress import IPv4Address, IPv6Address
from time import sleep
from typing import Any, Generator, Union

from .utils import add_help


@add_help
class NetLib:

    def help(): ...

    @staticmethod
    def cidr(address: str, netmask_or_prefix: int | str) -> dict[str, Any] | ValueError:
        """
        根据指定的网络和前缀长度计算CIDR信息。

        Args:
        - network: 字符串，指定的网络地址，可以是IPv4或IPv6格式。
        - prefix: 整数，网络的前缀长度。

        Returns:
        - 字典，包含CIDR信息，包括主机数量、地址范围、网络地址、广播地址和子网掩码。
        - 如果参数无效，返回ValueError异常。

        Examples:
        >>> print(Net.cidr('192.168.1.0', 24))

        >>> print(Net.cidr('240e::0', 64))
        """

        try:
            ip_type: ipaddress.IPv6Network | ipaddress.IPv4Network = (
                ipaddress.IPv6Network if ":" in address else ipaddress.IPv4Network
            )
            cidr = ip_type(f"{address}/{netmask_or_prefix}")

            # 输出网络地址、子网掩码、广播地址等信息
            host_count = cidr.num_addresses - 2
            range = (
                str(cidr.network_address + 1) + "-" + str(cidr.broadcast_address - 1)
            )
            network_address = str(cidr.network_address)
            broadcast_address = str(cidr.broadcast_address)
            netmask = str(cidr.netmask)
            prefixlen = cidr.prefixlen

            return {
                "host_count": host_count,
                "range": range,
                "network_address": network_address,
                "broadcast_address": broadcast_address,
                "prefixlen": prefixlen,
                "netmask": netmask,
            }

        except ValueError as e:
            return e

    @staticmethod
    def ipaddress_generator(
        number: int, max: int | None = None, is_ipv6: bool = False, **kwargs
    ) -> Generator[list[str], Any, None]:
        """
        生成指定数量的IP地址序列。

        Args:
        - number: 每次生成的IP地址数量。
        - max: 生成IP地址的最大数量, 为空时使用 number。
        - is_ipv6: 是否生成IPv6地址，默认生成IPv4地址。
        - **kwargs: 可选参数，用于指定IP地址的a、b、c、d、e、f、g、h初始值。

        Returns:
        - Generator: 生成器，每次返回number个IP地址组成的列表，直到生成max个IP地址为止。

        Examples:
        >>> for i in NetLib.ipaddress_generator(10, max=100):
        ...    print(";".join(map(lambda x: f"http://{x}", i)))

        >>> ip_list = [ip for sublist in NetLib.ipaddress_generator(10, a=100) for ip in sublist]

        """
        a: int = kwargs.get("a", 1)
        b: int = kwargs.get("b", 1)
        c: int = kwargs.get("c", 1)
        d: int = kwargs.get("d", 1)
        e: int = kwargs.get("e", 1)
        f: int = kwargs.get("f", 1)
        g: int = kwargs.get("g", 1)
        h: int = kwargs.get("h", 1)

        if max is None:
            max = number

        if is_ipv6 is False:
            a_range: int = 256
            b_range: int = 256
            c_range: int = 256
            d_range: int = 255
            ip_list: list = []
            count: int = 0
            for _ in range(1, max + 1):
                ip_list.append(ipaddress.IPv4Address(f"{a}.{b}.{c}.{d}").compressed)
                count += 1
                d += 1

                if d == d_range:
                    d = 1
                    c = c + 1
                    if c == c_range:
                        c = 1
                        b = b + 1
                        if b == b_range:
                            b = 1
                            a = a + 1
                            if a == a_range:
                                a = 1

                if len(ip_list) % number == 0:
                    yield ip_list
                    ip_list = []
                if max == count and len(ip_list) != 0:
                    yield ip_list
        else:
            a_range: int = 65536
            b_range: int = 65536
            c_range: int = 65536
            d_range: int = 65536
            e_range: int = 65536
            f_range: int = 65536
            g_range: int = 65536
            h_range: int = 65536
            ip_list: list = []
            count: int = 0
            for _ in range(1, max + 1):
                ip_list.append(
                    ipaddress.IPv6Address(
                        f"{a:04x}:{b:04x}:{c:04x}:{d:04x}:{e:04x}:{f:04x}:{g:04x}:{h:04x}"
                    ).compressed
                )
                count += 1
                h += 1

                if h == h_range:
                    h = 1
                    g = g + 1
                    if g == g_range:
                        g = 1
                        f = f + 1
                        if f == f_range:
                            f = 1
                            e = e + 1
                            if e == e_range:
                                e = 1
                                d = d + 1
                                if d == d_range:
                                    d = 1
                                    c = c + 1
                                    if c == c_range:
                                        c = 1
                                        b = b + 1
                                        if b == b_range:
                                            b = 1
                                            a = a + 1
                                            if a == a_range:
                                                a = 1

                if len(ip_list) % number == 0:
                    yield ip_list
                    ip_list = []
                if max == count and len(ip_list) != 0:
                    yield ip_list

    @staticmethod
    def generate_ip_list(
        number: int,
        *,
        is_ipv6: bool = False,
        ip_str: str | None = None,
        v6_exploded: bool = False,
    ) -> list[str]:
        """
        生成指定数量的IP地址字符串列表。

        参数:
        - number: 生成IP地址的数量。
        - is_ipv6: 指示是否生成IPv6地址。
        - ip_str: 指定IP地址字符串，如果提供了该参数，则使用该字符串作为起始IP地址。
        - v6_exploded: 指示生成的IPv6地址是否以展开格式表示。

        返回值:
        - 一个包含生成的IP地址字符串的列表。

        示例:
        >>> print(NetLib.generate_ip_list(3, ip_str="192.168.1.1"))

        >>> print(NetLib.generate_ip_list(3, is_ipv6=True, ip_str="2001:db8::1"))

        """
        if is_ipv6:
            if ip_str is None:
                ipv6 = int(ipaddress.ip_address("2001:db8:0:42:0:8a2e:370:1"))
            else:
                ipv6 = int(ipaddress.ip_address(ip_str))

            if v6_exploded:
                return [ipaddress.IPv6Address(ipv6 + i).exploded for i in range(number)]
            return [ipaddress.IPv6Address(ipv6 + i).compressed for i in range(number)]

        if ip_str is None:
            ipv4 = int(ipaddress.ip_address("1.0.0.1"))
        else:
            ipv4 = int(ipaddress.ip_address(ip_str))
        return [ipaddress.IPv4Address(ipv4 + i).compressed for i in range(number)]

    @staticmethod
    def is_ip_in_range(
        ip_str: str,
        subnet_str: Union[str, None] = None,
        *,
        src_ip: str | None = None,
        dst_ip: str | None = None,
    ) -> bool:
        """检查 IP 地址是否在指定的子网或 IP 范围内。

        参数:
        - ip_str: 要检查的 IP 地址。
        - subnet_str: 可选的子网（CIDR 形式或 IP 范围，格式为 'start_ip-end_ip'）。
        - src_ip: 可选的起始 IP 地址，当没有提供 `subnet_str` 时使用。
        - dst_ip: 可选的结束 IP 地址，当没有提供 `subnet_str` 时使用。

        返回值:
        - 如果 IP 地址在指定范围内，则返回 True;否则返回 False。

        示例:
        - is_ip_in_range("192.168.1.10", "192.168.1.0/24")
        - is_ip_in_range("192.168.1.10", "192.168.1.1-192.168.1.20")
        - is_ip_in_range("192.168.1.10", src_ip="192.168.1.1", dst_ip="192.168.1.20")

        - is_ip_in_range("2a00::110:1", "2a00::110:0/116")
        - is_ip_in_range("2a00::110:1", "2a00::110:1-2a00::110:fff")
        """
        ip: IPv4Address | IPv6Address = ipaddress.ip_address(ip_str)

        if subnet_str:
            if "/" in subnet_str:
                ip_network = ipaddress.ip_network(subnet_str, strict=False)
                return ip in ip_network
            elif "-" in subnet_str:
                start_ip_str, end_ip_str = map(str.strip, subnet_str.split("-"))
                start_ip, end_ip = ipaddress.ip_address(
                    start_ip_str
                ), ipaddress.ip_address(end_ip_str)
                return start_ip <= ip <= end_ip

        if src_ip and dst_ip:
            start_ip, end_ip = ipaddress.ip_address(src_ip), ipaddress.ip_address(
                dst_ip
            )
            return start_ip <= ip <= end_ip

        return False

    @staticmethod
    def mac_generator(
        number: int, max: int = 10, **kwargs
    ) -> Generator[list, Any, None]:
        """
        生成MAC地址
        """
        a: int = kwargs.get("a", 255)
        b: int = kwargs.get("b", 255)
        c: int = kwargs.get("c", 255)
        d: int = kwargs.get("d", 1)
        e: int = kwargs.get("e", 1)
        f: int = kwargs.get("f", 1)
        a_range: int = 256
        b_range: int = 256
        c_range: int = 256
        d_range: int = 256
        e_range: int = 256
        f_range: int = 256
        count = 0
        mac_list = []
        for _ in range(1, max + 1):
            mac_list.append(f"{a:02x}:{b:02x}:{c:02x}:{d:02x}:{e:02x}:{f:02x}")
            count += 1
            f += 1
            if f == f_range:
                f = 1
                e = e + 1
                if e == e_range:
                    e = 1
                    d = d + 1
                    if d == d_range:
                        d = 1
                        c = c + 1
                        if c == c_range:
                            c = 1
                            b = b + 1
                            if b == b_range:
                                b = 1
                                a = a + 1
                                if a == a_range:
                                    a = 1
            if len(mac_list) % number == 0:
                yield mac_list
                mac_list = []
            if max == count and len(mac_list) != 0:
                yield mac_list



import datetime
import random
import string
import warnings

from .utils import Singleton, add_help


@add_help
class Faker(Singleton):
    """伪造常用数据，计划在0.1.20删除"""

    def __init__(self):
        warnings.warn(
            "Faker 即将弃用删除，计划在0.1.20删除。改用 Person",
            DeprecationWarning,
            stacklevel=2,
        )

    def help(): ...

    @property
    def __areaCodeDict(self) -> dict:
        """区域文件地址"""
        from .data._internal_utils import data_dict

        return data_dict

    def __areaCode(self, dict) -> int:
        """功能：随机生产一个区域码"""
        code_list = list(dict.keys())
        length = len(code_list) - 1
        i = random.randint(0, length)
        return code_list[i]

    @property
    def __birthDay(self) -> str:
        """功能：随机生成1930年之后的出生日期"""
        d1 = datetime.datetime.strptime("1930-01-01 00:00:00", "%Y-%m-%d %H:%M:%S")
        d2 = datetime.datetime.now()
        delta = d2 - d1
        dys = delta.days
        i = random.randint(0, dys)
        dta = datetime.timedelta(days=i)
        bhday = d1 + dta
        return bhday.strftime("%Y%m%d")

    @property
    def __ordrNum(self) -> tuple[str, int]:
        """功能：随机生成3位序列号"""
        od_num = random.randint(100, 999)  # 随机生成100到999之间的位为数据
        sex = od_num % 2
        return str(od_num), sex

    def __check(self, id_num) -> str:
        """功能：生成校验码"""
        i = 0
        count = 0
        weight = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]  # 权重项
        check_code = {
            "0": "1",
            "1": "0",
            "2": "X",
            "3": "9",
            "4": "8",
            "5": "7",
            "6": "6",
            "7": "5",
            "8": "5",
            "9": "3",
            "10": "2",
        }  # 校验码映射
        for i in range(len(id_num)):
            count = count + int(id_num[i]) * weight[i]
        return check_code[str(count % 11)]  # 算出校验码

    @property
    def info(self) -> dict:
        """随机数据，用于测试目的。

        `info` 是一个属性，用于生成用于测试目的的随机数据。

        返回值:
            包含随机生成的个人信息的字典，例如姓名、性别、年龄、职业等。
        """
        area_code_dt = self.__areaCodeDict  # 生成字典
        area_cd = self.__areaCode(area_code_dt)  # 生成区域码
        area_cd_name = area_code_dt[area_cd]  # 获取区域名称
        birth_dy = self.__birthDay  # 生成出生日期
        (ordNum, sex) = self.__ordrNum  # 生成顺序号和性别
        check_code = self.__check((area_cd + birth_dy + ordNum))  # 生产校验码
        id_card = area_cd + birth_dy + ordNum + check_code  # 拼装身份证号

        gender = "女" if int(id_card[16]) % 2 == 0 else "男"
        # 计算年龄
        birth_date = datetime.datetime.strptime(birth_dy, "%Y%m%d")
        current_date = datetime.datetime.now()
        age = (
            current_date.year
            - birth_date.year
            - (
                (current_date.month, current_date.day)
                < (birth_date.month, birth_date.day)
            )
        )

        return {
            "姓名": self.name,
            "性别": gender,
            "年龄": age,
            "职业": self.occupation,
            "民族": self.ethnicities,
            "学校": self.school,
            "宗教信仰": self.religion,
            "手机号": self.phone_number,
            "座机号": self.home_phone_number,
            "地址": f"{area_cd_name}{self.housing}",
            "小区": self.housing,
            "身份证号": id_card,
            "MAC": self.mac_address,
            "护照号": self.passport_number,
            "纬度": self.latitude,
            "经度": self.longitude,
            "IP地址": self.ipv4,
            "国家": self.country(),
            "婚姻状态": self.marital_status,
            "证件类型": self.certificate_type,
            "学历": self.edu,
        }

    @property
    def edu(self):
        """学历"""
        return random.choice(
            ["博士", "硕士", "本科", "大专", "中专", "高中", "初中", "小学", "研究生"]
        )

    @property
    def certificate_type(self):
        """证件类型"""
        return random.choice(
            [
                "身份证",
                "居民身份证",
                "士官证",
                "军官证",
                "学生证",
                "驾驶证",
                "驾照",
                "护照",
                "港澳通行证",
                "营业执照",
                "组织机构代码证",
                "台胞证",
                "文职干部证",
                "部队离退休证",
                "香港特区护照/身份证明",
                "澳门特区护照/身份证明",
                "台湾居民来往大陆通行证",
                "境外永久居住证",
                "户口薄",
            ]
        )

    @property
    def marital_status(self):
        """婚姻状态"""
        return random.choice(["未婚", "已婚", "离婚", "丧偶", "离异"])

    @property
    def passport_number(self) -> str:
        """护照号"""
        return "D" + "".join(random.choices("0123456789", k=8))

    @property
    def gender(self) -> str:
        """性别

        如果要生成和身份证号一致的数据，应该使用：:func:`Faker().info`。

        """
        return random.choice(["男", "女"])

    @property
    def address(self) -> str:
        """地址

        如果要生成和身份证号一致的数据，应该使用：:func:`Faker().info`。

        """
        # return self.info['地址']

        from .data._addr import addr_info_list

        return random.choice(addr_info_list)

    @property
    def shop_sign(self) -> str:
        """店铺名称

        Returns:
            str: _description_
        """
        from .data._shop_sign import shop_sign_list

        return random.choice(shop_sign_list)

    @property
    def id_area(self) -> str:
        """身份证号"""
        return self.info["身份证号"]

    @property
    def mac_address(self) -> str:
        """MAC地址"""
        mac_parts = [
            random.randint(0x00, 0xFF) for _ in range(6)
        ]  # 生成六个随机的十六进制数
        # 使用冒号分隔，也可以使用 '-' 或其他分隔符
        mac_address = ":".join(["%02x" % part for part in mac_parts])
        return mac_address

    @property
    def phone_number(self) -> int:
        """手机号"""
        prefix = random.choice(
            [
                "130",
                "131",
                "132",
                "133",
                "134",
                "135",
                "136",
                "137",
                "138",
                "139",
                "150",
                "151",
                "152",
                "153",
                "155",
                "156",
                "157",
                "158",
                "159",
                "180",
                "181",
                "182",
                "183",
                "184",
                "185",
                "186",
                "187",
                "188",
                "189",
            ]
        )
        # 随机生成电话号的后缀
        suffix = "".join(
            random.choice(["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"])
            for _ in range(8)
        )
        # 将前缀和后缀组合起来，生成电话号
        return int(prefix + suffix)

    @property
    def home_phone_number(self) -> str:
        """座机号码"""
        return f"057{random.choice(string.digits)}-{random.choice(string.digits[1:])}{''.join(random.choice(string.digits) for _ in range(7))}"

    @property
    def latitude(self) -> float:
        """纬度

        N（北） 表示北半球。
        S（南） 表示南半球。
        """
        return f"{round(random.uniform(-90, 90), 6)}°{random.choice(['N','S'])}"

    @property
    def longitude(self) -> float:
        """经度

        E（东） 表示东经。
        W（西） 表示西经。
        """
        return f"{round(random.uniform(-180, 180), 6)}°{random.choice(['E','W'])}"

    @property
    def occupation(self) -> str:
        """职业"""
        return random.choice(
            [
                "教师",
                "工人",
                "记者",
                "演员",
                "作曲家",
                "架构师",
                "营养师",
                "鼓手",
                "厨师",
                "医生",
                "护士",
                "司机",
                "军人",
                "律师",
                "商人",
                "会 计",
                "店员",
                "出纳员",
                "作家",
                "导游",
                "警察",
                "歌手",
                "画家",
                "裁缝",
                "翻译官",
                "法官",
                "保安",
                "花匠",
                "服务员",
                "清洁工",
                "建筑师",
                "理发师",
                "采购员",
                "消防员",
                "机修工",
                "推销员",
                "魔术师",
                "模特",
                "邮递员",
                "售货员",
                "救生员",
                "运动员",
                "工程师",
                "飞行员",
                "管理员",
                "机械师",
                "经纪人",
                "歌唱家",
                "审计员",
                "漫画家",
                "园艺师",
                "科学家",
                "主持人",
                "调酒师",
                "化妆师",
                "艺术家",
                "糕点师",
                "甜品师",
                "外交官",
                "舞蹈家",
                "箭术",
                "溜冰",
                "弹钢琴",
                "古筝手",
                "钢琴家",
                "设计师",
                "CEO",
                "机 长",
                "记者",
                "赛车手",
                "教练",
                "兽医",
                "特警",
                "按摩",
            ]
        )

    @property
    def ipv4(self) -> str:
        """随机的欧洲国家 IP 地址"""
        europe_ip_prefixes = [
            "1",
            "2",
            "5",
            "25",
            "31",
            "37",
            "46",
            "62",
            "77",
            "78",
            "79",
            "80",
            "81",
            "82",
            "83",
            "84",
            "85",
            "86",
            "87",
            "88",
            "89",
            "90",
            "91",
            "92",
            "93",
            "94",
            "95",
            "109",
            "176",
            "178",
            "185",
            "188",
            "193",
            "194",
            "195",
            "212",
            "213",
            "217",
        ]
        random_ip = random.choice(europe_ip_prefixes)
        for _ in range(3):
            random_ip += f".{random.randint(0, 255)}"
        return random_ip

    @property
    def school(self) -> str:
        """学校名称"""
        return random.choice(
            [
                "麻省理工学院",
                "牛津大学",
                "斯坦福大学",
                "剑桥大学",
                "哈佛大学",
                "加州理工学院",
                "帝国理工学院",
                "苏黎世联邦理工大学",
                "伦敦大学学院",
                "芝加哥大学",
            ]
        )

    @property
    def name(self) -> str:
        """姓名"""
        surnames: list[str] = [
            "赵",
            "钱",
            "孙",
            "李",
            "周",
            "吴",
            "郑",
            "王",
            "冯",
            "陈",
            "褚",
            "卫",
            "蒋",
            "沈",
            "韩",
            "杨",
            "朱",
            "秦",
            "尤",
            "许",
            "何",
            "吕",
            "施",
            "张",
            "孔",
            "曹",
            "严",
            "华",
            "金",
            "魏",
            "陶",
            "姜",
            "戚",
            "谢",
            "邹",
            "喻",
            "柏",
            "水",
            "窦",
            "章",
            "云",
            "苏",
            "潘",
            "葛",
            "奚",
            "范",
            "彭",
            "郎",
            "鲁",
            "韦",
            "昌",
            "马",
            "苗",
            "凤",
            "花",
            "方",
            "俞",
            "任",
            "袁",
            "柳",
            "酆",
            "鲍",
            "史",
            "唐",
            "费",
            "廉",
            "岑",
            "薛",
            "雷",
            "贺",
            "倪",
            "汤",
            "滕",
            "殷",
            "罗",
            "毕",
            "郝",
            "邬",
            "安",
            "常",
            "乐",
            "于",
            "时",
            "傅",
            "皮",
            "卞",
            "齐",
            "康",
            "伍",
            "余",
            "元",
            "卜",
            "顾",
            "孟",
            "平",
            "黄",
            "郭",
            "轩辕",
            "公孙",
            "慕容",
            "司马",
        ]

        given_names: list[str] = [
            "伟",
            "芳",
            "娜",
            "秀英",
            "敏",
            "静",
            "丽",
            "强",
            "磊",
            "军",
            "洋",
            "勇",
            "艳",
            "杰",
            "娟",
            "博",
            "文",
            "涛",
            "慧",
            "明",
            "建国",
            "丽丽",
            "媛",
            "子涵",
            "子轩",
            "浩然",
            "昊然",
            "浩",
        ]

        surname: str = random.choice(surnames)
        given_name: str = random.choice(given_names)

        full_name: str = surname + given_name

        return full_name

    @property
    def religion(self) -> str:
        """宗教信仰"""
        return random.choice(
            [
                "道教",
                "佛教",
                "伊斯兰教",
                "基督教",
                "印度教",
                "犹太教",
                "婆罗门教",
                "耆那教",
            ]
        )

    @property
    def age(self) -> int:
        """年龄

        如果要生成和身份证号一致的数据，应该使用：:func:`Faker().info`。

        """
        return random.randint(16, 101)

    @property
    def ethnicities(self):
        """民族"""
        chinese_ethnicities = [
            "汉族",
            "蒙古族",
            "回族",
            "藏族",
            "维吾尔族",
            "苗族",
            "彝族",
            "壮族",
            "布依族",
            "朝鲜族",
            "满族",
            "侗族",
            "瑶族",
            "白族",
            "土家族",
            "哈尼族",
            "哈萨克族",
            "傣族",
            "黎族",
            "傈僳族",
            "佤族",
            "畲族",
            "高山族",
            "拉祜族",
            "水族",
            "东乡族",
            "纳西族",
            "景颇族",
            "柯尔克孜族",
            "土族",
            "达斡尔族",
            "仫佬族",
            "羌族",
            "布朗族",
            "撒拉族",
            "毛南族",
            "仡佬族",
            "锡伯族",
            "阿昌族",
            "普米族",
            "塔吉克族",
            "怒族",
            "乌孜别克族",
            "俄罗斯族",
            "鄂温克族",
            "崩龙族",
            "保安族",
            "裕固族",
            "京族",
            "塔塔尔族",
            "独龙族",
            "鄂伦春族",
            "赫哲族",
            "门巴族",
            "珞巴族",
            "基诺族",
        ]
        return random.choice(chinese_ethnicities)

    @property
    def housing(self):
        """小区"""
        from .data._housing import housing

        return random.choice(housing)

    def country(self, en: bool = False):
        """国家名"""
        from .data._country import en_country, zh_country

        if en:
            return random.choice(en_country)
        return random.choice(zh_country)

    @property
    def industry(self) -> str:
        """行业"""
        industry_list: list[str] = [
            "金融",
            "教育",
            "通信",
            "信息安全",
            "物流",
            "能源",
            "医疗",
            "军工",
        ]
        return random.choice(industry_list)



import os
import shutil
import sys
from pathlib import Path

from .utils import add_help


@add_help
class PathLib:
    """
    处理文件相关的功能
    """

    def help(): ...

    @staticmethod
    def clear_directory(path):
        """
        清空指定目录下的所有文件和子目录。

        Args:
            path:要清空的目录
        """
        if not os.path.exists(path):
            sys.exit("要清空的目录不存在")

        for filename in os.listdir(path):
            file_path = os.path.join(path, filename)
            if os.path.isfile(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)

    @staticmethod
    def del_key_directory(path: str, keywords: str | list[str], confirm: bool = False):
        """
        清理指定目录中包含特定关键词的子目录。

        Args:
            path: str - 要清理的目录路径。
            keywords: str | list[str] - 要查找和处理的关键词或关键词列表。
            confirm: bool - 删除是不可逆的，默认仅打印找到的子目录路径。如果为True，则删除找到的子目录。

        Raises:
            KeyError - 如果关键词列表为空，则抛出此异常。

        Examples:
            >>> del_key_directory('C:/Users/user/Documents', 'example')
            >>> del_key_directory('C:/Users/user/Documents', ['example', 'test'])
            >>> del_key_directory('C:/Users/user/Documents', ['example', 'test'], confirm=True)
        """

        keys = []

        if len(keywords) < 1:
            raise KeyError(f"要查找的目录名称不能是空！")

        if isinstance(keywords, str):
            keys.append(keywords)

        if isinstance(keywords, list):
            keys.extend(keywords)

        for dirname, subdir, filenames in os.walk(path):
            for dir_name in subdir:
                if dir_name in keys:
                    dir_path = os.path.join(dirname, dir_name)
                    if confirm:
                        shutil.rmtree(dir_path)
                    else:
                        print(dir_path)

    @staticmethod
    def del_key_file(path: str, keywords: str, confirm: bool = False) -> None:
        """
        清理指定目录下包含特定关键字的文件。

        Args:
            path: 字符串，指定要搜索的目录路径。
            keywords: 字符串，指定要查找的关键字。
            confirm: 布尔值，默认为False。如果设置为True，则实际删除找到的文件；
                    如果为False，则只打印将要删除的文件名。

        Examples:
            >>> del_key_file('C:/Users/user/Documents', 'example')
            >>> del_key_file('C:/Users/user/Documents', 'example', confirm=True)
        """
        file_list = [
            os.path.join(dirname, filename)
            for dirname, subdir, filenames in os.walk(path)
            for filename in filenames
        ]

        for target_file in file_list:
            if keywords in target_file:
                if confirm:
                    os.remove(target_file)
                else:
                    print(f"删除文件：{target_file}")

    @staticmethod
    def rename_files(path: str, old_string, new_string, confirm=False):
        """
        替换文件名中的指定字符串。

        该函数遍历指定路径下的所有文件，如果文件名包含旧字符串，

        则将其替换为新字符串，并执行重命名操作。如果confirm参数为False，

        则不会真的重命名文件，只会打印出将要执行的操作。

        Args:
            path: str - 文件夹路径，从中搜索文件进行重命名。
            old_string: str - 需要被替换的旧字符串。
            new_string: str - 用于替换旧字符串的新字符串。
            confirm: bool - 是否真的执行重命名操作，默认为False，只打印将要执行的操作。

        Examples:
            >>> rename_files('C:/Users/user/Documents', 'example', 'test')
            >>> rename_files('C:/Users/user/Documents', 'example', 'test', confirm=True)
        """

        for dirname, subdir, filenames in os.walk(path):
            for filename in filenames:
                if old_string in filename:
                    old_file_path = os.path.join(dirname, filename)
                    new_file_name = filename.replace(old_string, new_string)
                    new_file_path = os.path.join(dirname, new_file_name)

                    if confirm:
                        os.rename(old_file_path, new_file_path)
                    else:
                        s_len = len(old_file_path.encode("gbk"))
                        print("*" * s_len)
                        print(f"{old_file_path}\n{new_file_path}")
                        print("*" * s_len)

    @staticmethod
    def del_key_row(file: str, key: str) -> None:
        """
        删除文件中和 key 相等的行

        Args:
            file : str
            key : str
        """
        with open(file, "r", encoding="utf8") as f:
            lines: list[str] = f.readlines()
        with open(file, "w", encoding="utf8") as f:
            for line in lines:
                if line.strip("\n") != key:
                    f.write(line)

    @staticmethod
    def get_dir_file_list(path: str) -> list:
        """
        获取path下所有文件的路径，包括子目录。
        """
        return [
            os.path.join(root, fn)
            for root, dirs, files in os.walk(path)
            for fn in files
        ]

    @staticmethod
    def mkdir_dir(path) -> None:
        """
        如果目录不存在就创建

        Args:
            path:目录
        """
        if not os.path.exists(path):
            os.mkdir(path)

    @staticmethod
    def get_project_path(project_name: str) -> str:
        """
        根据项目名称获取项目根目录绝对路径
        """
        curPath: str = os.path.abspath(os.path.dirname(__file__))
        return curPath[: curPath.find(project_name) + len(project_name)]

    @staticmethod
    def is_none_then_mkdir(path: str | Path) -> Path:
        """如果指定的路径不存在，则创建该路径及其所有父路径。

        参数:
        - path: 字符串或Path类型，表示需要检查和可能创建的路径。

        返回值:
        - Path类型，表示处理后的路径对象。
        """

        if not isinstance(path, Path):
            path = Path(path)  # 将字符串类型的路径转换为Path对象

        if not path.exists():
            path.mkdir(parents=True)  # 如果路径不存在，则创建该路径及其所有父路径
        return path

    @staticmethod
    def move_videos_to_root(
        root_path, use_parent_dir_as_prefix=False, remove_empty_dirs=True
    ):
        """把源目录的子目录中的视频文件移动到源目录下

        参数:

        - use_parent_dir_as_prefix：设置为 True 时使用上一级目录名作为文件名前缀；设置为 False 时保持原文件名。

        - remove_empty_dirs：设置为 True 时，移动完文件后删除原目录（前提是目录为空）；设置为 False 时不删除目录。
        """

        # 定义视频文件格式
        video_extensions = {".mp4", ".mkv", ".avi", ".mov", ".flv", ".wmv", ".webm"}
        moved_files_count = 0  # 初始化移动文件计数器

        # 遍历指定目录及其子目录
        for dirpath, _, filenames in os.walk(root_path, topdown=False):
            for filename in filenames:
                # 检查文件扩展名是否在视频格式列表中
                if any(filename.lower().endswith(ext) for ext in video_extensions):
                    source_path = os.path.join(dirpath, filename)

                    # 如果文件已经在根目录中，则跳过
                    if dirpath == root_path:
                        continue

                    # 使用上一级目录名作为文件名前缀（根据可选参数）
                    if use_parent_dir_as_prefix:
                        parent_dir_name = os.path.basename(os.path.dirname(source_path))
                        new_filename = f"{parent_dir_name}_{filename}"
                    else:
                        new_filename = filename

                    destination_path = os.path.join(root_path, new_filename)

                    # 若文件名冲突，则自动重命名
                    if os.path.exists(destination_path):
                        base, ext = os.path.splitext(new_filename)
                        count = 1
                        while os.path.exists(destination_path):
                            destination_path = os.path.join(
                                root_path, f"{base}_{count}{ext}"
                            )
                            count += 1

                    # 移动文件到目标目录
                    shutil.move(source_path, destination_path)
                    moved_files_count += 1  # 计数移动的文件
                    print(f"Moved: {source_path} -> {destination_path}")

            # 删除空目录（根据可选参数）
            if remove_empty_dirs and dirpath != root_path and not os.listdir(dirpath):
                os.rmdir(dirpath)
                print(f"Removed empty directory: {dirpath}")

        print(f"Total files moved: {moved_files_count}")


@add_help
class DirLib:
    """处理目录相关功能"""

    def __init__(self, directory_path: str | Path):
        # 创建 Path 对象
        self.directory: Path = Path(directory_path)

        # 使用 exists() 方法检查目录是否存在,不存在时新建
        if not self.directory.exists():
            self.directory.mkdir(parents=True)

    def help(): ...

    @property
    def is_directory_empty(self) -> bool:
        """
        检查目录是否为空。

        无需参数，方法内部会使用实例属性 `directory` 来代表需要检查的目录。
        目录下有空目录时也会判断为非空

        返回值:
            bool: 如果目录为空，则返回 True；否则返回 False。
        """

        # 使用 `any()` 函数和目录迭代器判断目录中是否至少有一个文件或子目录
        return not any(self.directory.iterdir())

    def rename_non_empty_directory(self, new_path: str | Path = None):
        """
        重命名非空目录。

        如果指定的目录不为空，则为其生成一个新的名称，如果指定新的路径，则将目录移动到该路径下，并在原路径上重命名。
        如果新的路径不存在，则会创建该路径。

        参数:
        - new_path: str | Path 类型，指定目录重命名后的新路径。如果为 None，则只重命名不移动目录。

        返回值:
        - Path 类型，表示重命名后的目录路径。如果目录为空，则返回 False。
        """
        # 检查目录是否为空，不为空则进行重命名
        if not self.is_directory_empty:

            serial_number: int = 1  # 初始化序列号，用于生成新的目录名称
            while True:
                # 生成新的目录名称，如果存在则递增序列号
                new_directory_name = f"{self.directory.stem}-{serial_number}"

                if new_path is None:
                    # 如果未指定新路径，则在原路径下重命名
                    new_directory_path = self.directory.parent / new_directory_name
                else:
                    # 如果指定了新路径，则在新路径下重命名
                    new_path = Path(new_path)
                    PathLib.is_none_then_mkdir(new_path)  # 确保新路径存在
                    new_directory_path: Path = new_path / new_directory_name

                if new_directory_path.exists():
                    serial_number += 1  # 如果新名称已存在，则递增序列号并重试
                    continue
                else:
                    break  # 找到可用的新名称，退出循环

            # 执行目录重命名操作
            self.directory.rename(new_directory_path)

            return new_directory_path
        else:
            # 目录为空，不进行重命名
            return False

    @staticmethod
    def delete_glob_file(dir_path: Path | str, file_name: str):
        """
        删除指定目录下匹配的文件

        Example:
            delete_glob_file(Path("./"), "*.json")
        """
        try:
            for file in dir_path.glob("qemu_info_*.json"):
                file.unlink()
        except Exception as e:
            return e

        return True



"""
辅助测试的一些常用功能
"""

from ._date import DateLib
from ._faker import Faker
from ._net import NetLib
from ._path import DirLib, PathLib
from ._person import Person
from ._rand import Random

__all__: list[str] = [
    "DateLib",
    "Faker",
    "PathLib",
    "DirLib",
    "Random",
    "NetLib",
    "Person",
]
